
<!DOCTYPE html>

<html>
    <body>
        {% if name %}
        <p>{{ name }}</p>
        {% endif %}

        {{ form.errors }}
        <form method="POST" enctype="multipart/form-data">
            {{ form.hidden_tag() }}
            <p>
                {{ form.name.label }}
                {{ form.name }}
            </p>
            <p>
                {{ form.submit }}
            </p>
        </form>
    </body>
</html>
